@@ -33,6 +33,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <glib.h>
+#include <glib/gprintf.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 
@@ -74,6 +75,7 @@ struct UICStruct {
 	GtkWidget *contactInfosCheckBox;
 	GtkWidget *sendSMSButton;
 	GtkWidget *smsLabel;
+	GtkWidget *getSmsMenuItem;
 	gchar *smsText;
 	osso_context_t *osso_context;
 	gboolean contactInfosAvailable;
@@ -88,6 +90,7 @@ struct UICStruct {
 	gboolean pollingPhone;
 	gboolean noConfig;
 	gint callDetected;
+	gboolean smsDeliveryReport;
 };
 typedef struct UICStruct UIComponents;
 
@@ -105,7 +108,7 @@ static void dial (UIComponents *uic);
 static void sendSMSToTree (UIComponents *uic);
 void dialNumberFromSearch(UIComponents * uic, gboolean gsm);
 void dialNumber(UIComponents * uic, gchar *number);
-static void sendSMS (gchar *number, gchar *text);
+static void sendSMS (gchar *number, gchar *text, gboolean smsDeliveryReport);
 
 static void  quit_event( UIComponents *uic, guint callback_action, GtkWidget *menu_item	) {
 	GConfClient *gconf_client = NULL;
@@ -113,6 +116,7 @@ static void  quit_event( UIComponents *uic, guint callback_action, GtkWidget *me
 	gconf_client = gconf_client_get_default ();
 	gconf_client_set_bool(gconf_client, GCONF_TRANSFORM_PLUS_KEY, uic->transformPlus, NULL);
 	gconf_client_set_bool(gconf_client, GCONF_IGNORE_CONTACTINFOS_KEY, uic->ignoreCI, NULL);
+	gconf_client_set_bool(gconf_client, GCONF_SMS_DELIVERY_REPORT_KEY, uic->smsDeliveryReport, NULL);
 	gconf_client_set_bool(gconf_client, GCONF_NOTIFY_CALL_KEY, uic->getCalls, NULL);
 	gconf_client_set_bool(gconf_client, GCONF_NOTIFY_SMS_KEY, uic->getSms, NULL);
 	gconf_client_suggest_sync (gconf_client, NULL);
@@ -284,7 +288,7 @@ gboolean useContactInfos(UIComponents *uic) {
 	
 	if (uic->ignoreCI) {
 		useCI = FALSE;
-		c = gtk_entry_get_text(GTK_ENTRY(uic->searchEntry));
+		c = (gchar *) gtk_entry_get_text(GTK_ENTRY(uic->searchEntry));
 		if (c) {
 			len = strlen(c);
 			for (i=0; i<len; i++) {
@@ -328,11 +332,14 @@ gn_error answer(int lowlevel_id) {
 }
 
 
-void notify_sms(gn_sms *message, struct gn_statemachine *state, UIComponents *uic) {
+gn_error notify_sms(gn_sms *message, struct gn_statemachine *state, void *callback_data) {
 	/* in this callback function you can't use those libgnokii functions that send a packet */
 	gchar *descr;
 	GtkWidget *note;
 	gint resultCode;
+	UIComponents *uic;
+	
+	uic = (UIComponents *)callback_data;
 	
 	g_printf("\n===== ===== Incoming SMS ===== =====\n");
 	g_printf("===== from %s =====\n", message->remote.number);
@@ -358,13 +365,18 @@ void notify_sms(gn_sms *message, struct gn_statemachine *state, UIComponents *ui
 		break;
 	}
 	gtk_widget_destroy (note);
+	
+	return GN_ERR_NONE;
 
 }
 
-void notify_call(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, UIComponents *uic) {
+void notify_call(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, void *callback_data) {
 	gchar *descr;
 	GtkWidget *note;
 	gint resultCode;
+	UIComponents *uic;
+	
+	uic = (UIComponents *)callback_data;
 	
 	/* in this callback function you can't use those libgnokii functions that send a packet */
 	
@@ -465,7 +477,7 @@ void on_menu_getCalls(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
 gboolean getSms(GtkWidget *topLevel, UIComponents *uic) {
 	gn_data		data;
 	gn_error	error = GN_ERR_NONE;
-    gchar       *text;
+	gchar       *text;
 	
 	if (uic->getSms) {
 		if (!uic->phoneOpened) {
@@ -519,6 +531,42 @@ void on_menu_getSms(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
 	}
 }
 
+void on_menu_smsDeliveryReport(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
+	GtkWidget *note;
+	gint resultCode;
+	gchar *descr;
+	
+	uic->smsDeliveryReport = !(uic->smsDeliveryReport);
+	gtk_check_menu_item_set_active(checkMenuitem, uic->smsDeliveryReport);
+	
+	if (uic->smsDeliveryReport && !uic->getSms) {
+		descr = g_strdup(_("Would you like to activate SMS Notification ?"));
+	} else if (!uic->smsDeliveryReport && uic->getSms) {
+		descr = g_strdup(_("Would you like to de-activate SMS Notification ?"));
+	} else {
+		return;
+	}
+	note = hildon_note_new_confirmation(NULL, descr);
+	gtk_widget_show_all (note);
+
+	resultCode = gtk_dialog_run (GTK_DIALOG (note));
+	
+	switch (resultCode) {
+	case GTK_RESPONSE_OK:
+/*		gtk_check_menu_item_toggled (GTK_CHECK_MENU_ITEM(uic->getSmsMenuItem)); */
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(uic->getSmsMenuItem), !uic->getSms);
+/*		uic->getSms = !uic->getSms;
+		getSms(GTK_WIDGET(gtk_widget_get_toplevel(GTK_WIDGET(checkMenuitem))), uic);
+	*/
+		break;
+	default:
+		break;
+	}
+	gtk_widget_destroy (note);
+
+}
+
+
 GtkMenu *create_menus(UIComponents *uic) {
 	GtkWidget *menuBar;
 	GtkWidget *menuItem;
@@ -544,10 +592,15 @@ GtkMenu *create_menus(UIComponents *uic) {
 	gtk_widget_set_sensitive(menuItem, !uic->noConfig);
 	gtk_menu_append(menuBar, menuItem);
 	
-	menuItem = gtk_check_menu_item_new_with_label (_("Notify me on SMS reception"));
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menuItem), uic->getSms);
-	g_signal_connect (G_OBJECT (menuItem), "toggled", G_CALLBACK (on_menu_getSms), (gpointer) uic);
-	gtk_widget_set_sensitive(menuItem, !uic->noConfig);
+	uic->getSmsMenuItem = gtk_check_menu_item_new_with_label (_("Notify me on SMS reception"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(uic->getSmsMenuItem), uic->getSms);
+	g_signal_connect (G_OBJECT (uic->getSmsMenuItem), "toggled", G_CALLBACK (on_menu_getSms), (gpointer) uic);
+	gtk_widget_set_sensitive(uic->getSmsMenuItem, !uic->noConfig);
+	gtk_menu_append(menuBar, uic->getSmsMenuItem);
+	
+	menuItem = gtk_check_menu_item_new_with_label (_("SMS Delivery Report"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menuItem), uic->smsDeliveryReport);
+	g_signal_connect (G_OBJECT (menuItem), "toggled", G_CALLBACK (on_menu_smsDeliveryReport), (gpointer) uic);
 	gtk_menu_append(menuBar, menuItem);
 	
 	gtk_menu_append(menuBar, gtk_separator_menu_item_new());
@@ -872,31 +925,6 @@ void smsBufferChanged_cb(GtkTextBuffer *textBuffer, UIComponents * uic) {
 	
 }
 
-static gchar * cleanNumber(gchar * number) {
-	guint i, j;
-
-	if (number) {
-		for (i = j = 0; number[i]; i++) {
-			if (number[i] != ' ') {
-				number[j++] = number[i];
-			}
-		}
-		number[j] = '\0';
-	}
-	return number;
-}
-
-/*
-static void getPhoneInfos_cb(GtkButton *widget, gpointer data) {
-	gchar *text;
-	text = getPhoneInfos(gtk_widget_get_toplevel(GTK_WIDGET(widget)));
-	if (text) {
-		g_print("%s\n", text);
-		g_free(text);
-	}
-}
-*/
-
 static void showMessageSendSMS(GtkWidget *widget, gchar *number) {
 	gchar *text;
 	text = g_strdup_printf("Sending SMS to %s ...", number);
@@ -924,7 +952,7 @@ static void sendSMS_cb (GtkButton *widget, UIComponents *uic) {
 				return;
 			}
 			
-			sendSMS(normalizedNumber, text);
+			sendSMS(normalizedNumber, text, uic->smsDeliveryReport);
 			g_free(text);
 
 			/* tear down the connection */
@@ -937,7 +965,7 @@ static void sendSMS_cb (GtkButton *widget, UIComponents *uic) {
 
 }
 
-static void sendSMS (gchar *number, gchar *text) {
+static void sendSMS (gchar *number, gchar *text, gboolean smsDeliveryReport) {
 	gn_sms sms;
 	gn_error error;
 	gn_data data;
@@ -948,7 +976,7 @@ static void sendSMS (gchar *number, gchar *text) {
 	 * 	   255 * 153 = 39015 default alphabet characters */
 	int input_len, i, curpos = 0;
 
-	g_printf("sendSMS(number = %s, text = %s)\n", number, text);
+	g_printf("sendSMS(number = %s, text = %s, smsDeliveryReport = %d)\n", number, text, smsDeliveryReport);
 	
 	input_len = GN_SMS_MAX_LENGTH;
 
@@ -963,6 +991,9 @@ static void sendSMS (gchar *number, gchar *text) {
 		sms.remote.type = GN_GSM_NUMBER_Unknown;
 	}
 	
+	/* Delivery Report */
+	sms.delivery_report = smsDeliveryReport;
+	
 	data.message_center = calloc(1, sizeof(gn_sms_message_center));
 	data.message_center->id = 1;
 	if (gn_sm_functions(GN_OP_GetSMSCenter, &data, state) == GN_ERR_NONE) {
@@ -996,6 +1027,9 @@ static void sendSMS (gchar *number, gchar *text) {
 
 	if (error == GN_ERR_NONE) {
 		g_print("Send succeeded!\n");
+		if (smsDeliveryReport) {
+			g_print("SMS Delivery Report : %d\n",sms.status);
+		}
 	} else {
 		g_print("SMS Send failed (%s)\n", gn_error_print(error));
 	}
@@ -1012,7 +1046,7 @@ gboolean sendSMSToIter (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 	gtk_tree_model_get(model, iter, NUMBER_COLUMN, &number, -1);
 	normalizedNumber = normalizeNumber(number, uic->transformPlus);
 	showMessageSendSMS(GTK_WIDGET(uic->searchEntry), normalizedNumber);
-	sendSMS (normalizedNumber, text);
+	sendSMS (normalizedNumber, text, uic->smsDeliveryReport);
 	g_free(normalizedNumber);
 	g_free(text);
 	return FALSE;
@@ -1045,44 +1079,6 @@ gboolean clearSMSTextView_cb(GtkTextView *smsTextView, GdkEventFocus *event, UIC
 }
 
 
-/**
- * Create the menu items needed for the drop down menu.
- */
-/*
-void menu_init(HildonWindow *window) {
-    /* Create needed handles. */
-/*
-    GtkMenu *menu;
-    GtkWidget *submenu;
-    GtkWidget *menu_item;
-    printf("%s()\n", __PRETTY_FUNCTION__);
-
-    /* Get the menu of our view. */
-/*
-    menu = GTK_MENU(gtk_menu_new());
-
-    /* Create the menu items. */
-
-    /* The "Config" submenu. */
-/*
-    gtk_menu_append(menu, menu_item
-            = gtk_menu_item_new_with_label("configuration"));
-
-
-    /* We need to show menu items. */
-/*
-    gtk_widget_show_all(GTK_WIDGET(menu));
-
-    hildon_window_set_menu(HILDON_WINDOW(window), menu);
-
-    /* Connect the "Route" signals. */
-/*
-    g_signal_connect(G_OBJECT(menu_item), "activate",
-                      G_CALLBACK(getPhoneInfos_cb), NULL);
-
-}
-*/
-
 int main( int argc, char *argv[] ) {
 	    /* GtkWidget is the storage type for widgets */
 	GtkWidget *button;
@@ -1094,7 +1090,7 @@ int main( int argc, char *argv[] ) {
 	GtkTextBuffer *buffer;
 	GConfClient *gconf_client = NULL;
 	gboolean gnokiiFile = FALSE;
-    gchar *text;
+	gchar *text;
 	
 #ifdef GNOKII_GCONF
 	gchar ** gnokiiCfg;
@@ -1123,6 +1119,7 @@ int main( int argc, char *argv[] ) {
 	gconf_client = gconf_client_get_default ();
 	uic->transformPlus = gconf_client_get_bool(gconf_client, GCONF_TRANSFORM_PLUS_KEY, NULL);
 	uic->ignoreCI = gconf_client_get_bool(gconf_client, GCONF_IGNORE_CONTACTINFOS_KEY, NULL);
+	uic->smsDeliveryReport = gconf_client_get_bool(gconf_client, GCONF_SMS_DELIVERY_REPORT_KEY, NULL);
 	uic->getCalls= gconf_client_get_bool(gconf_client, GCONF_NOTIFY_CALL_KEY, NULL);
 	uic->getSms= gconf_client_get_bool(gconf_client, GCONF_NOTIFY_SMS_KEY, NULL);
 
@@ -1168,7 +1165,7 @@ int main( int argc, char *argv[] ) {
 	/* Initialize maemo application */
 	uic->osso_context = osso_initialize(APPLICATION_DBUS_SERVICE, VERSION, FALSE, NULL);
 
-	g_printf("apres osso_init %x\n", uic->osso_context);
+	g_printf("apres osso_init %p\n", uic->osso_context);
 	            
 	/* Check that initialization was ok */
 	if (uic->osso_context == NULL) {
